import 'package:flutter/material.dart';
import 'package:mobile_ui/src/utils/function.dart';

import 'circular_loading.dart';

typedef ButtonBuilder = Function(CustomButtonStyle style, VoidCallback? onPressed, bool loading,
    BuildContext context);

enum ButtonType {
  raised,
  flat,
  outline,
  text,
}

class CustomButtonStyle {
  Color? color;
  Color? textColor;
  ButtonType buttonType;
  double? height;
  double? fontSizeFactor;
  double? fontSizeDelta;
  EdgeInsetsGeometry? padding;
  double? radiusWidth;
  double? elevation;
  Color? shadowColor;
  Color? borderColor;

  CustomButtonStyle(
      {required this.buttonType,
      this.color,
      this.textColor,
      this.height,
      this.fontSizeFactor,
      this.fontSizeDelta,
      this.padding,
      this.radiusWidth,
      this.elevation,
      this.shadowColor,
      this.borderColor});

  CustomButtonStyle copyWith({double? elevation, Color? shadowColor}) {
    return CustomButtonStyle(
        buttonType: buttonType,
        color: color,
        textColor: textColor,
        height: height,
        fontSizeDelta: fontSizeDelta,
        fontSizeFactor: fontSizeFactor,
        padding: padding,
        radiusWidth: radiusWidth,
        elevation: elevation ?? this.elevation,
        shadowColor: shadowColor ?? this.shadowColor);
  }

  CustomButtonStyle apply(
      {Color? color,
      Color? textColor,
      double? height,
      double? fontSizeFactor,
      double? elevation,
      double? fontSizeDelta,
      EdgeInsetsGeometry? padding,
      double? radiusWidth,
      Color? shadowColor,
      Color? borderColor}) {
    this.color = color ?? this.color;
    this.textColor = textColor ?? this.textColor;
    this.height = height ?? this.height;
    this.fontSizeFactor = fontSizeFactor ?? this.fontSizeFactor;
    this.elevation = elevation ?? this.elevation;
    this.fontSizeDelta = fontSizeDelta ?? this.fontSizeDelta;
    this.padding = padding ?? this.padding;
    this.radiusWidth = radiusWidth ?? this.radiusWidth;
    this.borderColor = borderColor ?? this.borderColor;
    return this;
  }

  CustomButtonStyle.raised(
      {Color? color,
      Color? textColor,
      double? height,
      double? fontSizeFactor,
      double? elevation,
      double? fontSizeDelta,
      EdgeInsetsGeometry? padding,
      double? radiusWidth,
      Color? shadowColor})
      : this(
            buttonType: ButtonType.raised,
            color: color,
            textColor: textColor,
            height: height,
            fontSizeDelta: fontSizeDelta,
            fontSizeFactor: fontSizeFactor,
            padding: padding,
            radiusWidth: radiusWidth,
            elevation: elevation,
            shadowColor: shadowColor);

  CustomButtonStyle.flat(
      {Color? color,
      Color? textColor,
      double? height,
      double? fontSizeFactor,
      double? fontSizeDelta,
      EdgeInsetsGeometry? padding,
      double? radiusWidth})
      : this(
            buttonType: ButtonType.flat,
            color: color,
            textColor: textColor,
            height: height,
            fontSizeDelta: fontSizeDelta,
            fontSizeFactor: fontSizeFactor,
            padding: padding,
            radiusWidth: radiusWidth);

  CustomButtonStyle.outline(
      {Color? color,
      Color? textColor,
      double? height,
      double? fontSizeFactor,
      double? fontSizeDelta,
      EdgeInsetsGeometry? padding,
      double? radiusWidth,
      Color? borderColor})
      : this(
            buttonType: ButtonType.outline,
            color: color,
            textColor: textColor,
            height: height,
            fontSizeDelta: fontSizeDelta,
            fontSizeFactor: fontSizeFactor,
            padding: padding,
            radiusWidth: radiusWidth,
            borderColor: borderColor);

  CustomButtonStyle.text(
      {Color? color,
      Color? textColor,
      double? height,
      double? fontSizeFactor,
      double? fontSizeDelta,
      EdgeInsetsGeometry? padding})
      : this(
            buttonType: ButtonType.outline,
            color: color,
            textColor: textColor,
            height: height,
            fontSizeDelta: fontSizeDelta,
            fontSizeFactor: fontSizeFactor,
            padding: padding);
}

class Button extends StatelessWidget {
  final CustomButtonStyle? buttonStyle;
  final VoidCallback? onPressed;
  final AsyncVoidCallback? onAsyncPressed;
  final String? text;
  final String? loadingText;
  final double? fontSizeFactor;
  final double? fontSizeDelta;
  final IconData? icon;
  final TextAlign textAlign;
  final MainAxisAlignment align;
  final bool? isShowLoading;
  final double? height;
  final double? width;

  const Button({
    this.onPressed,
    this.onAsyncPressed,
    this.buttonStyle,
    this.text,
    this.loadingText,
    this.fontSizeFactor,
    this.fontSizeDelta,
    this.icon,
    this.textAlign = TextAlign.center,
    this.align = MainAxisAlignment.center,
    this.isShowLoading,
    this.height,
    this.width,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    ThemeData theme = Theme.of(context);
    final style = buttonStyle ??
        CustomButtonStyle.flat(color: theme.primaryColor, radiusWidth: 2.0);
    switch (style.buttonType) {
      case ButtonType.text:
        return _buildButton(context, style, _buildTextButton);
      case ButtonType.outline:
        return _buildButton(context, style, _buildOutlineButton);
      case ButtonType.flat:
        return _buildButton(context, style, _buildElevatedButton);
      case ButtonType.raised:
      default:
        return _buildButton(
            context,
            style.copyWith(
                elevation: style.elevation ?? 10,
                shadowColor: style.shadowColor ?? theme.shadowColor),
            _buildElevatedButton);
    }
  }

  Widget _buildButton(BuildContext context, CustomButtonStyle style, ButtonBuilder buttonBuilder) {
    final showLoading = isShowLoading ?? (onAsyncPressed != null);
    if (showLoading) {
      var loading = false;
      return StatefulBuilder(builder: (_context, setState) {
        return buttonBuilder(
            style,
            (onPressed == null && onAsyncPressed == null)
                ? null
                : () async {
                    if (loading) return;
                    setState(() {
                      loading = true;
                    });
                   await _handleFocus();
                    if (onPressed != null) {
                      onPressed!();
                    }
                    if (onAsyncPressed != null) {
                      await onAsyncPressed!();
                    }
                    setState(() {
                      loading = false;
                    });
                  },
            loading,
            _context);
      });
    } else {
      return buttonBuilder(style, () async {
                    await _handleFocus();
                    if (onPressed != null) {
                      onPressed!();
                    }
                    if (onAsyncPressed != null) {
                      await onAsyncPressed!();
                    }
                  }, false, context);
    }
  }

  Future<void> _handleFocus() async {
    final effectiveFocusNode =
        FocusManager.instance.primaryFocus;
    if (effectiveFocusNode != null &&
        effectiveFocusNode.hasFocus) {
      effectiveFocusNode.unfocus();
      await Future.delayed(Duration(milliseconds: 20));
    }
  }

  Widget _buildTextButton(CustomButtonStyle style, VoidCallback? onPressed,
      bool loading, BuildContext context) {
    final theme = Theme.of(context);
    final textWidget = _buildTextWidget(loading, theme, onPressed, style);
    return Row(
      mainAxisAlignment: align,
      children: [
        if (icon != null)
          Icon(icon,
              color: onPressed == null ? theme.disabledColor : style.textColor),
        TextButton(onPressed: onPressed, child: textWidget),
      ],
    );
  }

  Text _buildTextWidget(bool loading, ThemeData theme, VoidCallback? onPressed,
      CustomButtonStyle style) {
    return Text(
      (loading && loadingText != null && loadingText!.isNotEmpty)
          ? loadingText!
          : text!,
      textAlign: textAlign,
      style: theme.textTheme.button!.apply(
          color: onPressed != null ? style.textColor : Colors.black26,
          fontSizeFactor: fontSizeFactor ?? style.fontSizeFactor ?? 1.0,
          fontSizeDelta: fontSizeDelta ?? style.fontSizeDelta ?? 0.0),
    );
  }

  Widget _buildOutlineButton(CustomButtonStyle style, VoidCallback? onPressed,
      bool loading, BuildContext context) {
    final theme = Theme.of(context);
    style.apply(
        textColor: style.textColor ?? theme.primaryColor,
        borderColor: style.borderColor ?? theme.primaryColor);
    final textWidget = _buildTextWidget(loading, theme, onPressed, style);
    final ButtonStyle outlineButtonStyle = OutlinedButton.styleFrom(
      primary: style.color,
      fixedSize: Size(width ?? double.infinity, height ?? style.height ?? 36),
      side: BorderSide(color: style.borderColor ?? Colors.black26),
      padding: style.padding,
      shape: RoundedRectangleBorder(
        borderRadius:
            BorderRadius.all(Radius.circular(style.radiusWidth ?? 2.0)),
      ),
    );
    return OutlinedButton(
      onPressed: onPressed,
      style: outlineButtonStyle,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          if (loading)
            CircularLoading(
              color: style.textColor,
            ),
          if (icon != null)
            Icon(icon,
                color:
                    onPressed == null ? theme.disabledColor : style.textColor),
          textWidget
        ],
      ),
    );
  }

  Widget _buildElevatedButton(CustomButtonStyle style, VoidCallback? onPressed,
      bool loading, BuildContext context) {
    final theme = Theme.of(context);
    final textWidget = _buildTextWidget(loading, theme, onPressed, style);
    final ButtonStyle raisedButtonStyle = ElevatedButton.styleFrom(
      onPrimary: style.textColor,
      elevation: style.elevation,
      shadowColor: style.shadowColor,
      primary: style.color,
      fixedSize: Size(width ?? double.infinity, height ?? style.height ?? 36),
      padding: style.padding,
      shape: RoundedRectangleBorder(
        borderRadius:
            BorderRadius.all(Radius.circular(style.radiusWidth ?? 2.0)),
      ),
    );

    return ElevatedButton(
      onPressed: onPressed,
      style: raisedButtonStyle,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          if (loading)
            CircularLoading(
              color: style.textColor,
            ),
          if (icon != null)
            Icon(icon,
                color:
                    onPressed == null ? theme.disabledColor : style.textColor),
          textWidget
        ],
      ),
    );
  }
}
