import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:mobile_ui/mobile_ui.dart';
import 'package:mobile_ui/src/controls/search_input.dart';

class DropDownFormInput<T> extends FormInputBase<T> {
  final bool canClear;
  final bool allowSearch;
  final bool allowViewHeader;
  final List<DropDownOption<T>> options;

  static Future<void> showList<T>(
      BuildContext context,
      T? initialValue,
      List<DropDownOption<T>> options,
      ValueChanged<T?> onChanged,
      bool allowViewHeader,
      bool allowSearch,
      String? hintText) async {
    final value = await showModalBottomSheet<T>(
      context: context,
      builder: (_) => BottomSheetWidget<T>(
        options: options,
        allowSearch: allowSearch,
        allowViewHeader: allowViewHeader,
        hintText: hintText,
      ),
    );
    if (value != initialValue) onChanged(value);
  }

  DropDownFormInput(
      {T? initialValue,
      ValueController<T>? controller,
      this.canClear = false,
      this.allowSearch = true,
      this.allowViewHeader = false,
      required this.options,
      String? labelText,
      String? hintText,
      bool readOnly = false,
      bool enabled = true,
      bool isRequired = false,
      ValueChanged<T?>? onChanged,
      FormInputBackgroundStyle? backgroudStyle =
          defalutFormInputBackgroundStyle,
      bool? isDense,
      FormFieldSetter<T>? onSaved,
      ValueValidator<T?>? validator,
      AutovalidateMode? autovalidateMode,
      TextAlign labelAlign = TextAlign.left,
      FieldOrientation orientation = FieldOrientation.vertical,
      EdgeInsetsGeometry contentPadding = const EdgeInsets.all(8),
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
      ControlPosition controlPosition = ControlPosition.trailing,
      double spacingFlactor = 1,
      double? labelWidth,
      bool? textSoftWrap = false,
      int errorMaxLines = 1})
      : assert(initialValue == null || controller == null),
        super(
            initialValue: initialValue,
            labelWidth: labelWidth,
            errorMaxLines: errorMaxLines,
            onSaved: onSaved,
            validator: validator,
            readOnly: readOnly,
            enabled: enabled,
            spacingFlactor: spacingFlactor,
            autovalidateMode: autovalidateMode,
            orientation: orientation,
            controlPosition: controlPosition,
            isDense: isDense,
            labelText: labelText,
            labelAlign: labelAlign,
            valueController: controller,
            isRequired: isRequired,
            contentPadding: contentPadding,
            onChanged: onChanged,
            margin: margin,
            builder: (field) {
              var widget = field.widget as DropDownFormInput;
              final state = field as FormInputStateBase<T?>;
              final theme = Theme.of(field.context);
              var text = '';
              if (field.value != null) {
                final selectedOption = options
                    .firstWhere((element) => element.value == field.value);
                text = selectedOption.text;
              }
              final icon = (canClear && field.value != null)
                  ? widget.buildIcon(theme, Icons.clear, onTap: () {
                      state.changeValue(null);
                    })
                  : widget.buildIcon(theme, Icons.arrow_drop_down);
              Widget textWidget;
              if (hintText != null && hintText.isNotEmpty && (text.isEmpty)) {
                textWidget = Text(hintText,
                    style: theme.inputDecorationTheme.hintStyle,
                    softWrap: false);
              } else {
                textWidget =
                    widget.buildText(theme, text, softWrap: textSoftWrap);
              }
              final child = InkWell(
                  onTap: (readOnly || enabled == false)
                      ? null
                      : () {
                          showList<T>(
                              field.context,
                              field.value,
                              options,
                              state.changeValue,
                              allowViewHeader,
                              allowSearch,
                              hintText);
                        },
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[Expanded(child: textWidget), icon],
                  ));
              return widget.buildField(field, child,
                  backgroudStyle: backgroudStyle);
            });
}

class DropDownOption<T> {
  T value;
  String text;

  DropDownOption({required this.value, required this.text});
}

class BottomSheetWidget<T> extends StatefulWidget {
  final List<DropDownOption<T>> options;
  bool allowSearch = true;
  bool allowViewHeader = false;
  String? hintText;

  BottomSheetWidget(
      {required this.options,
      required this.allowSearch,
      required this.allowViewHeader,
      this.hintText});

  @override
  _BottomSheetState<T> createState() => _BottomSheetState<T>();
}

class _BottomSheetState<T> extends State<BottomSheetWidget<T>> {
  String _searchText = '';
  late List<DropDownOption<T>> _searchOptions;

  @override
  void initState() {
    super.initState();
    _searchOptions = widget.options;
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.symmetric(
        horizontal: uiSpacing(2),
        vertical: uiSpacing(2),
      ),
      child: widget.options.isEmpty
          ? Container(
              child: Loading(),
            )
          : Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.start,
              children: <Widget>[
                widget.allowViewHeader
                    ? Label(
                        widget.hintText ?? '',
                        color: Theme.of(context).primaryColor,
                      )
                    : Container(),
                widget.allowViewHeader
                    ? SizedBox(
                        height: uiSpacing(4),
                      )
                    : Container(),
                widget.allowSearch
                    ? SearchInput(
                        // hintText: widget.hintText,
                        hintText: widget.hintText,
                        onSubmitted: (value) {
                          if (_searchText == value) return;
                          _searchText = value;
                          setState(() {
                            _searchOptions = (value.isEmpty)
                                ? widget.options
                                : widget.options
                                    .where(
                                      (v) => v.text.toLowerCase().contains(
                                            value.toLowerCase(),
                                          ),
                                    )
                                    .toList();
                          });
                        },
                      )
                    : Container(),
                widget.allowSearch
                    ? SizedBox(
                        height: uiSpacing(4),
                      )
                    : Container(),
                Expanded(
                  child: ListView.builder(
                    physics: BouncingScrollPhysics(),
                    itemCount:
                        _searchOptions == null ? 0 : _searchOptions.length + 1,
                    itemBuilder: (context, index) {
                      if (_searchOptions.isEmpty) {
                        return Label(
                          "Sorry, we couldn't find any results matching \'$_searchText\'.",
                          type: BaseTextType.subtitle1,
                          maxLines: 3,
                        );
                      } else {
                        if (index < _searchOptions.length) {
                          final option = _searchOptions[index];
                          return Column(
                            children: [
                              ListTile(
                                title: Label(option.text),
                                onTap: () {
                                  Navigator.of(context).pop(option.value);
                                },
                              ),
                              Divider(
                                height: 1,
                                color: Colors.black26,
                              )
                            ],
                          );
                        }
                        return Container();
                      }
                    },
                  ),
                )
              ],
            ),
    );
  }
}
